from django.core.management.base import BaseCommand
from django.utils.translation import gettext as _

from django_afip import models


class Command(BaseCommand):
    help = _("Loads fixtures with metadata from AFIP.")
    requires_migrations_checks = True

    def handle(self, *args, **options):
        models.load_metadata()
